/**
 * Maa ewolucja - Gra ewolucyjno spoeczna
 *
 *	Klasa odpowiadajca za wczytanie tekstw do programu.
 *
 * @author Adam Makowski
 */

package settings;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Scanner;

public class Dictionary extends FileManager
{
	/**
	 * Pliki istotne w przypadku wczytywania tekstw.
	 */
	public static final String DIR_TO_DICTIONARIES = "dictionaries/";
	public static final String FILE_NAME = "text";
	public static final String SETTING_FILE_NAME = "settings.txt";
	/**
	 * Jzyk - domylnie, angielski.
	 */
	public static String lang = "eng";

	/**
	 * Zbir par kod tekstu - tekst.
	 */
	private static HashMap<String, String> dictionary = new HashMap<String, String>();

	/**
	 * Wczytaj jzyk z pliku.
	 */
	public static void loadLanguage()
	{
		String tmp = null;
		File file;

		file = new File(SETTING_FILE_NAME);

		if(!file.exists())
			return;

	    Scanner in = null;
		try
		{
			in = new Scanner(file);
			while(in.hasNextLine())
			{
				tmp=in.nextLine();
				if(tmp.startsWith("lang"))
				{
					tmp = tmp.substring(6);
					lang = tmp.replace(" ", "");
				}
			}
			if(tmp == null)
				tmp = "eng";
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Wczytaj plik sownika.
	 */
	public static void loadDictionary()
	{
		loadLanguage();
		String tmp;
		File file;

		file = new File(DIR_TO_DICTIONARIES + FILE_NAME + "." + lang);

		if(!file.exists())
			return;

	    Scanner in = null;
		try
		{
			in = new Scanner(file);

			tmp=in.nextLine();
			if(!tmp.equals("QBS DICTIONARY"))
				throw new DictionaryNotFoundException();

			tmp=in.nextLine();
			if(!tmp.equals(lang.toUpperCase()))
				throw new DictionaryNotFoundException();

			tmp=in.nextLine();
			if(!tmp.isEmpty())
				throw new DictionaryNotFoundException();

			fillDictionary(in);

		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (DictionaryNotFoundException e)
		{
			e.printStackTrace();
		}
		finally
		{
			if(!in.equals(null))
				in.close();
		}
	}

	/**
	 * Wypenij sownik na podstawie pliku.
	 * @param in
	 */
	private static void fillDictionary(Scanner in)
	{
		String record, code, text;
		Integer indexOfSpace;
		while(in.hasNextLine())
		{
			record = in.nextLine();

			if(isComment(record))
				continue;

			indexOfSpace = record.indexOf(" ");

			if(indexOfSpace == -1)
				continue;

			code = record.substring(0, indexOfSpace);
			text = record.substring(indexOfSpace + 1);
			dictionary.put(code, text);
		}
	}

	/**
	 * Zwrc teskt odpowiadajcy zadanemu kodowi. Jeeli nie ma takiego w programie, zwrc kod.
	 * @param code Kod tekstu
	 */
	public static String getText(String code)
	{
		String text = dictionary.get(code);

		if(text == null)
			text = code;

		return text;
	}
	
	/**
	 * Zwr jzyk programu
	 */
	public static String getLang()
	{
		return lang;
	}
}
