/**
 * Maa ewolucja - Gra ewolucyjno spoeczna
 *
 *	Klasa panelu do repreznetacji struktury populacji.
 *
 * @author Adam Makowski
 */
package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import settings.Dictionary;
import settings.Settings;
import javax.swing.SwingConstants;
import logic.MutatedThief;
import logic.MutatedProducer;
import logic.PopulationMember;
import logic.Thief;
import logic.Producer;

@SuppressWarnings("serial")
public class PopulationProportionPane extends JPanel
{
	/**
	 * Strategie wystpujce w symulacji.
	 */
	public enum Person{THIEF, PRODUCER, MUTATED_THIEF, MUTATED_PRODUCER};
	
	private JScrollBar scroll;
	private JLabel title;
	
	private int sizeX;
	private int sizeY;
	private int width = 100;
	
	/**
	 * Runda od ktrej bd rysowane statystyki.
	 */
	private int firstElement;
	
	/**
	 * Ile osobnikw bdzie wywietlane w panelu.
	 */
	private int personsToShow = 0;
	
	/**
	 * Aktualny cykl.
	 */
	private int selectedCycle = 0;
	
	/**
	 * Flaga - czy symulacja trwa czy jest zatrzymana.
	 */
	private boolean run = false;

	/**
	 * Tryb pracy panelu. Jeeli proportionMode == true, pokazuje proporcje midzy osobnikami rnych strategii w grupie ktra przesza by selekcje. 
	 * W przeciwnym przypadku, pokazuje pierwszych x osobnikw populacji posortowanej wzgldem zdobytych punktw.
	 */
	private boolean proportionMode = false;

	/**
	 * Jak dua cz populacji przesza by selekcje.
	 */
	private int howMuchSelect = 0;

	/**
	 * Lista pokazujca procentowy udzia poszczeglnych strategii w rnych cyklach.
	 */
	List<ProportionRow> population = new ArrayList<ProportionRow>();
	/**
	 * Lista pokazujca kolejno osobnikw w populacji wzgldem zdobytych punktw w rnych cyklach.
	 */
	List<Person[]> sortedPopulation = new ArrayList<Person[]>();

	/**
	 * Konstruktor.
	 */
	PopulationProportionPane() {
		sizeX = (int) getSize().getWidth();
		sizeY = (int) getSize().getHeight();

		setBackground(Settings.getColor("CHARTS_COLOR"));
		
		title = new JLabel("<html><p width=\"200\"><center>" + Dictionary.getText("POPULATION_PROPORTION_TITLE")
				+ "</center></p></html>");
		title.setFont(new Font("Tahoma", Font.BOLD, 40));
		title.setHorizontalAlignment(SwingConstants.CENTER);
		title.setForeground(Settings.getColor("SIMULATION_WINDOW_FONT"));
		setLayout(new BorderLayout());
		add(title);

		scroll = new JScrollBar(JScrollBar.HORIZONTAL);
		scroll.setBorder(null);
		scroll.addAdjustmentListener(new AdjustmentListener() {
			@Override
			public void adjustmentValueChanged(AdjustmentEvent e) {
				repaint();
			}
		});
		scroll.setBackground(Settings.getColor("SCROLLS_SIMULATION"));

		add(scroll, BorderLayout.SOUTH);
	}

	public void setProportionMode(boolean mode)
	{
		proportionMode = mode;
		repaint();
	}

	public void putProportionRow(ProportionRow record) {
		population.add(record);
	}

	public void putSortedPopulation(List<PopulationMember> populationIn) {
		personsToShow = sizeY < populationIn.size() ? sizeY : populationIn.size();

		Person[] populationTmp = new Person[personsToShow];
		for(int i = 0; i < populationIn.size() && i < personsToShow; i++)
		{
			if(populationIn.get(populationIn.size()-1-i) instanceof MutatedThief)
				populationTmp[i] = Person.MUTATED_THIEF;
			else if(populationIn.get(populationIn.size()-1-i) instanceof Thief)
				populationTmp[i] = Person.THIEF;
			else if(populationIn.get(populationIn.size()-1-i) instanceof MutatedProducer)
				populationTmp[i] = Person.MUTATED_PRODUCER;
			else if(populationIn.get(populationIn.size()-1-i) instanceof Producer)
				populationTmp[i] = Person.PRODUCER;
		}
		sortedPopulation.add(populationTmp);
	}

	/**
	 * Wyczy listy.
	 */
	public void clearList() {
		population.clear();
		sortedPopulation.clear();
		repaint();
	}

	public void lockScroll(boolean lock) {
		scroll.setEnabled(!lock);
		if (!lock) {
			if (population.size() > width)
				scroll.setMaximum(population.size() - width);
			else
				scroll.setMaximum(0);
			scroll.setValue(population.size() - width);
		}
	}
	/**
	 * Policz od ktrej rundy naley wywietla dane.
	 */
	private void setFirstElement() {

		if (scroll.isEnabled()) {
			firstElement = scroll.getValue();
		} else {
			if (population.size() > width)
				firstElement = population.size() - width;
			else
				firstElement = 0;
		}
	}

	/**
	 * Narysuj wykres w trybie proporcjonalnym.
	 * @param g
	 */
	private void paintProportion(Graphics g)
	{
		Map<Integer, Long> selectionLines = new HashMap<Integer, Long>();
		Graphics2D g2d = (Graphics2D) g;

		Line2D thiefsLine;
		Line2D producerLine;
		Line2D mutatedThiefsLine;
		Line2D mutatedProducersLine;

		sizeX = (int) getSize().getWidth();
		sizeY = (int) getSize().getHeight();

		sizeY = sizeY - (int) scroll.getSize().getHeight();
		width = sizeX;

		title.setVisible(population.size() == 0);

		setFirstElement();

		for (int i = 0, j = firstElement; i < width && j < population.size(); i++, j++) {
			double lvl0, lvl1, lvl2, lvl3, lvl4;

			lvl0 = 0.0;
			lvl1 = population.get(j).getMutatedProducers();
			lvl2 = lvl1 + population.get(j).getProducers();
			lvl3 = lvl2 + population.get(j).getThiefs();
			lvl4 = 1.0;

			mutatedThiefsLine = new Line2D.Double(i, sizeY * lvl4, i, sizeY * lvl3);

			thiefsLine = new Line2D.Double(i, sizeY * lvl3, i, sizeY * lvl2);

			producerLine = new Line2D.Double(i, sizeY * lvl2, i, sizeY * lvl1);

			mutatedProducersLine = new Line2D.Double(i, sizeY * lvl1, i, sizeY * lvl0);

			g.setColor(getColor(Settings.getColor("MUTATED_THIEF_COLOR_SIMULATION"), population.get(j).getNumberOfCycle()));
			g2d.draw(mutatedThiefsLine);

			g.setColor(getColor(Settings.getColor("THIEF_COLOR_SIMULATION"), population.get(j).getNumberOfCycle()));
			g2d.draw(thiefsLine);

			g.setColor(getColor(Settings.getColor("PRODUCER_COLOR_SIMULATION"), population.get(j).getNumberOfCycle()));
			g2d.draw(producerLine);

			g.setColor(getColor(Settings.getColor("MUTATED_PRODUCER_COLOR_SIMULATION"), population.get(j).getNumberOfCycle()));
			g2d.draw(mutatedProducersLine);

			if (population.get(j).isSelection()) {
				g.setColor(Settings.getColor("CYCLE_LINE"));
				g2d.draw(new Line2D.Double(i + 1, sizeY, i + 1, 0));
				selectionLines.put(i+2, population.get(j).getNumberOfCycle());
				i++;
			}
		}
		g.setColor(Settings.getColor("SELETION_TEXT"));
	    g.setFont(new Font("Tahoma", Font.BOLD, 15));
	    Iterator<Entry<Integer, Long>> iterator = selectionLines.entrySet().iterator();
	    while (iterator.hasNext()) {
	        Map.Entry<Integer, Long> selectionLine = (Map.Entry<Integer, Long> )iterator.next();

			g2d.drawString(selectionLine.getValue().toString(), selectionLine.getKey(), sizeY);
	        iterator.remove();
	    }
	}
	
	/**
	 * Metoda rozjaniajca kolor w wybranym cyklu.
	 * @param color Kolor
	 * @param numberOfCycle Numer cyklu
	 */
	private Color getColor(Color color, Long numberOfCycle)
	{
		int lighten = 150;
		int r,g,b;
		r = color.getRed() + lighten;
		if(r > 255)
			r = 255;
		g = color.getGreen() + lighten;
		if(g > 255)
			g = 255;
		b = color.getBlue() + lighten;
		if(b > 255)
			b = 255;
		
		if(numberOfCycle == selectedCycle)
			return new Color(r, g, b);
		return color;
	}

	/**
	 * Narysuj wykres w trybie pokazywania kolejnoci osobnikw.
	 * @param g
	 */
	protected void paintPopulation(Graphics g)
	{
		Map<Integer, Long> selectionLines = new HashMap<Integer, Long>();

		Graphics2D g2d = (Graphics2D) g;

		Line2D pixel = new Line2D.Double();

		sizeX = (int) getSize().getWidth();
		sizeY = (int) getSize().getHeight();

		sizeY = sizeY - (int) scroll.getSize().getHeight();
		width = sizeX;

		title.setVisible(sortedPopulation.size() == 0);

		setFirstElement();

		for (int i = 0, j = firstElement; i < width && j < sortedPopulation.size(); i++, j++) {
			for(int k = 0, m = 0; k < personsToShow && m < personsToShow; k++, m++)
			{
				switch(sortedPopulation.get(j)[k])
				{
				case THIEF:
					g.setColor(getColor(Settings.getColor("THIEF_COLOR_SIMULATION"), population.get(j).getNumberOfCycle()));
					break;
				case MUTATED_THIEF:
					g.setColor(getColor(Settings.getColor("MUTATED_THIEF_COLOR_SIMULATION"), population.get(j).getNumberOfCycle()));
					break;
				case PRODUCER:
					g.setColor(getColor(Settings.getColor("PRODUCER_COLOR_SIMULATION"), population.get(j).getNumberOfCycle()));
					break;
				case MUTATED_PRODUCER:
					g.setColor(getColor(Settings.getColor("MUTATED_PRODUCER_COLOR_SIMULATION"), population.get(j).getNumberOfCycle()));
					break;
				default:
					break;
				}

				pixel.setLine(i, m, i, m);
				g2d.draw(pixel);

				if(k == howMuchSelect)
				{
					g.setColor(Settings.getColor("SELETION_LINE"));
					m++;

					g2d.draw(new Line2D.Double(i, m, i, m));
				}
			}
			if (population.get(j).isSelection()) {
				g.setColor(Settings.getColor("CYCLE_LINE"));
				int lineY;
				if(sizeY>personsToShow)
					lineY= personsToShow;
				else
					lineY=sizeY;
				g2d.draw(new Line2D.Double(i + 1, lineY, i + 1, 0));
				selectionLines.put(i+2, population.get(j).getNumberOfCycle());

				i++;
			}
		}
		g.setColor(Settings.getColor("SELETION_TEXT"));
	    Iterator<Entry<Integer, Long>> iterator = selectionLines.entrySet().iterator();
	    g.setFont(new Font("Tahoma", Font.BOLD, 15));
	    while (iterator.hasNext()) {
	        Map.Entry<Integer, Long> selectionLine = (Map.Entry<Integer, Long> )iterator.next();
			g2d.drawString(selectionLine.getValue().toString(), selectionLine.getKey(), personsToShow);
	        iterator.remove();
	    }
		g.setFont(new Font("Tahoma", Font.BOLD, 25));
		g2d.drawString(Dictionary.getText("SELECTION_LINE_TEXT"), 0, howMuchSelect);
	}

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		if(proportionMode)
			paintProportion(g);
		else
			paintPopulation(g);
	}

	public void setHowMuchSelect(int howMuchSelect) {
		this.howMuchSelect = howMuchSelect;
	}

	/**
	 * Oznacz i skocz do cyklu wybranego poprzez kliknicie go na wykresie.
	 * @param number
	 */
	public void markCycle(int number)
	{
		selectedCycle = number+1;
		if(!run)
		{
			for(int i = 0; i < population.size(); i++)
			{
				if(population.get(i).getNumberOfCycle() == selectedCycle)
				{
					scroll.setValue(i == 0 ? 0 : i-1);
					break;
				}
			}
		}
		repaint();
	}

	public boolean isRun() {
		return run;
	}

	public void setRun(boolean run) {
		this.run = run;
		selectedCycle = 0;
	}
}